/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "MapEffectGroupModel.h"

// Sigmodr tree includes
#include "MapEffectModel.h"

// Sigmod includes
#include <sigmod/Map.h>
#include <sigmod/MapEffect.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Tree;

MapEffectGroupModel::MapEffectGroupModel(BaseModel* parent, Map* map) :
        GroupModel(parent, map, "Effects")
{
    for (int i = 0; i < map->effectCount(); ++i)
        addObject(map->effect(i));
}

QVariant MapEffectGroupModel::data(const int role) const
{
    if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Effect", this, SLOT(addObject()));
        return QVariant::fromValue(menu);
    }
    return GroupModel::data(role);
}

bool MapEffectGroupModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml) && (xml.doctype().name() == "MapEffect"))
            {
                addObject(qobject_cast<Map*>(m_object)->newEffect(xml.documentElement()));
                return true;
            }
        }
    }
    return false;
}

QString MapEffectGroupModel::types() const
{
    return "MapEffect";
}

void MapEffectGroupModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<Map*>(m_object)->newEffect();
    if (object->className() == "MapEffect")
        addChild(new MapEffectModel(this, qobject_cast<MapEffect*>(object)));
}

void MapEffectGroupModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<Map*>(m_object)->deleteEffect(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}
