/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "SoundGroupModel.h"

// Sigmodr tree includes
#include "SoundModel.h"

// Sigmod includes
#include <sigmod/Game.h>
#include <sigmod/Sound.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Tree;

SoundGroupModel::SoundGroupModel(BaseModel* parent, Game* game) :
        GroupModel(parent, game, "Sounds")
{
    for (int i = 0; i < game->soundCount(); ++i)
        addObject(game->sound(i));
}

QVariant SoundGroupModel::data(const int role) const
{
    if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Sound", this, SLOT(addObject()));
        return QVariant::fromValue(menu);
    }
    return GroupModel::data(role);
}

bool SoundGroupModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml) && (xml.doctype().name() == "Sound"))
            {
                addObject(qobject_cast<Game*>(m_object)->newSound(xml.documentElement()));
                return true;
            }
        }
    }
    return false;
}

QString SoundGroupModel::types() const
{
    return "Sound";
}

void SoundGroupModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<Game*>(m_object)->newSound();
    if (object->className() == "Sound")
        addChild(new SoundModel(this, qobject_cast<Sound*>(object)));
}

void SoundGroupModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<Game*>(m_object)->deleteSound(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}
