/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "SpeciesModel.h"

// Sigmodr tree includes
#include "SpeciesMoveModel.h"
#include "GroupModel.h"

// Sigmodr widget includes
#include <sigmodr/widgets/SpeciesUI.h>

// Sigmod includes
#include <sigmod/Game.h>
#include <sigmod/Species.h>
#include <sigmod/SpeciesMove.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Widgets;
using namespace Sigmodr::Tree;

SpeciesModel::SpeciesModel(BaseModel* parent, Species* species) :
        GroupObjectModel(parent, species)
{
    setupData();
}

SpeciesModel::~SpeciesModel()
{
    clearData();
}

QVariant SpeciesModel::data(int role) const
{
    if (role == Qt::DisplayRole)
        return qobject_cast<Species*>(m_object)->name();
    else if (role == BaseModel::XmlRole)
    {
        QDomDocument xml(m_object->className());
        xml.appendChild(m_object->save());
        return xml.toString();
    }
    else if (role == BaseModel::WidgetRole)
    {
        QWidget* widget = new SpeciesUI(qobject_cast<Species*>(m_object), NULL);
        return QVariant::fromValue(widget);
    }
    else if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Move", this, SLOT(addObject()));
        menu->addSeparator();
        menu->addAction("&Delete Species", this, SLOT(deleteSelf()));
        return QVariant::fromValue(menu);
    }
    return GroupObjectModel::data(role);
}

bool SpeciesModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml))
            {
                if (xml.doctype().name() == m_object->className())
                {
                    clearData();
                    m_object->load(xml.documentElement());
                    setupData();
                    return true;
                }
                else if (xml.doctype().name() == "SpeciesMove")
                {
                    addObject(qobject_cast<Species*>(m_object)->newMove(xml.documentElement()));
                    return true;
                }
            }
        }
    }
    return false;
}

QString SpeciesModel::types() const
{
    return type().append(QString(";SpeciesMove"));
}

void SpeciesModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<Species*>(m_object)->newMove();
    if (object->className() == "SpeciesMove")
        addChild(new SpeciesMoveModel(this, qobject_cast<SpeciesMove*>(object)));
}

void SpeciesModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<Species*>(m_object)->deleteMove(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}

void SpeciesModel::deleteSelf()
{
    qobject_cast<GroupModel*>(m_parent)->deleteObject(this);
}

void SpeciesModel::setupData()
{
    Species* species = qobject_cast<Species*>(m_object);
    for (int i = 0; i < species->moveCount(); ++i)
        addChild(new SpeciesMoveModel(this, species->move(i)));
}
