/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "SpriteGroupModel.h"

// Sigmodr tree includes
#include "SpriteModel.h"

// Sigmod includes
#include <sigmod/Game.h>
#include <sigmod/Sprite.h>

// KDE includes
#include <KMenu>

using namespace Sigmod;
using namespace Sigmodr::Tree;

SpriteGroupModel::SpriteGroupModel(BaseModel* parent, Game* game) :
        GroupModel(parent, game, "Sprites")
{
    for (int i = 0; i < game->spriteCount(); ++i)
        addObject(game->sprite(i));
}

QVariant SpriteGroupModel::data(const int role) const
{
    if (role == BaseModel::ContextMenuRole)
    {
        KMenu* menu = new KMenu;
        menu->addAction("&Add Sprite", this, SLOT(addObject()));
        return QVariant::fromValue(menu);
    }
    return GroupModel::data(role);
}

bool SpriteGroupModel::setData(const QVariant& value, int role)
{
    if (role == BaseModel::XmlRole)
    {
        QString data = value.toString();
        if (!data.isEmpty())
        {
            QDomDocument xml;
            if (loadFromData(data, &xml) && (xml.doctype().name() == "Sprite"))
            {
                addObject(qobject_cast<Game*>(m_object)->newSprite(xml.documentElement()));
                return true;
            }
        }
    }
    return false;
}

QString SpriteGroupModel::types() const
{
    return "Sprite";
}

void SpriteGroupModel::addObject(Object* object)
{
    if (!object)
        object = qobject_cast<Game*>(m_object)->newSprite();
    if (object->className() == "Sprite")
        addChild(new SpriteModel(this, qobject_cast<Sprite*>(object)));
}

void SpriteGroupModel::deleteObject(BaseModel* model)
{
    const int index = find(model);
    if (0 <= index)
    {
        qobject_cast<Game*>(m_object)->deleteSprite(index);
        m_objects[index]->deleteLater();
        m_objects.removeAt(index);
        childRowChanged(index);
    }
}
