/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

// Header include
#include "AuthorUI.h"
#include "AuthorUI_p.h"

// Sigmod includes
#include <sigmod/Author.h>

// KDE includes
#include <KLineEdit>

using namespace Sigmod;
using namespace Sigmodr::Widgets;

AuthorUI::AuthorUI(Author* author, QWidget* parent) :
        ObjectUI(author, parent),
        d(new Private(new Author(*author)))
{
    setWidget(d->makeWidgets(this));
}

void AuthorUI::apply()
{
    *qobject_cast<Author*>(m_object) = *d->m_author;
    ObjectUI::apply();
}

void AuthorUI::discard()
{
    *d->m_author = *qobject_cast<Author*>(m_object);
    d->resetGui();
    ObjectUI::discard();
}

AuthorUI::Private::Private(Author* author) :
        ObjectUIPrivate(author),
        m_author(author)
{
}

AuthorUI::Private::~Private()
{
    delete m_author;
}

QWidget* AuthorUI::Private::makeWidgets(ObjectUI* widget)
{
    QWidget *form = openUiFile(":/gui/author.ui", widget);
    ui_name = form->findChild<KLineEdit*>("varName");
    ui_email= form->findChild<KLineEdit*>("varEmail");
    ui_role = form->findChild<KLineEdit*>("varRole");
    connect(ui_name, SIGNAL(textChanged(QString)), this, SLOT(nameChanged(QString)));
    connect(ui_email, SIGNAL(textChanged(QString)), this, SLOT(emailChanged(QString)));
    connect(ui_role, SIGNAL(textChanged(QString)), this, SLOT(roleChanged(QString)));
    return form;
}

void AuthorUI::Private::resetGui()
{
    ui_name->setText(m_author->name());
    ui_email->setText(m_author->email());
    ui_role->setText(m_author->role());
}

void AuthorUI::Private::nameChanged(const QString& name)
{
    const int cursor = ui_name->cursorPosition();
    m_author->setName(name);
    ui_name->setCursorPosition(cursor);
}

void AuthorUI::Private::emailChanged(const QString& email)
{
    const int cursor = ui_email->cursorPosition();
    m_author->setEmail(email);
    ui_email->setCursorPosition(cursor);
}

void AuthorUI::Private::roleChanged(const QString& role)
{
    const int cursor = ui_role->cursorPosition();
    m_author->setRole(role);
    ui_role->setCursorPosition(cursor);
}
